#ifndef Slash_directory_h
#define Slash_directory_h

#include <string>
#include <vector>
#include <cstdlib> //getenv

#ifdef WIN32
#include <windows.h>
#include <direct.h>
#else //UNIX
#include <dirent.h>
#if defined(_POSIX_SOURCE)
#define REAL_DIR_ENTRY(dp) 1
#else
#define REAL_DIR_ENTRY(dp) (dp->d_ino != 0)
#endif
#include <unistd.h>
#endif

#include <sys/stat.h>

namespace Slash {
namespace directory {

inline bool pwd(std::string& a_pwd) {
  // Return current directory.
  a_pwd = "";
#define SLASH_MAXPATHLEN 1024
  char cwd[SLASH_MAXPATHLEN];
#ifdef WIN32
  //  driveletter = 0 means return the working directory for the default drive.
  if(::_getdcwd(0,cwd,SLASH_MAXPATHLEN)==NULL) return false;
#else
  if(::getcwd(cwd,SLASH_MAXPATHLEN)==NULL) return false;
#endif
#undef SLASH_MAXPATHLEN
  a_pwd = cwd;
  return true;
}

inline bool cd(const std::string& a_path){
  if(::chdir(a_path.c_str())!=0) return false;
  return true;
}

inline bool create(const std::string& a_name){
#ifdef WIN32
  return (::mkdir(a_name.c_str())==0 ? true : false); 
#else
  return (::mkdir(a_name.c_str(), 0755)==0 ? true : false); 
#endif
}

inline std::string home() {
#ifdef WIN32
  const char* env = ::getenv("HOMEDRIVE");
  std::string drive = env ? env : std::string("C:");
 {const char* env = ::getenv("HOMEPATH");
  if(!env) return drive+"\\";
  return drive+std::string(env);}
#else
  const char* env = ::getenv("HOME");
  return (env?env:"");
#endif
}

inline bool cd_home() {
  std::string s = home();
  if(s.empty()) return false;
  return cd(s);
}

//NOTE : SWIG : exists is also a method in Slash/Tools/file.h
//       (can be fix with a %rename(directory_exists))
//inline bool exists(const std::string& a_path){
//  struct stat finfo;
//  if (::stat(a_path.c_str(),&finfo) < 0) return false;
//  return true;
//}

//NOTE : SWIG : "is" is a Python keyword.
inline bool is_a(const std::string& a_path,bool& a_value) {
  a_value = false;
  struct stat finfo;
  if (::stat(a_path.c_str(),&finfo) < 0) return false;
  a_value = ( ((finfo.st_mode & S_IFMT) == S_IFDIR) ? true : false);
  return true;
}

inline bool mkcd(const std::string& a_name) {
  // a_name should be a single directory name, and not a file system path.
  // Then it must not contain : ., .., /, \ etc...
  bool is;
  if(!directory::is_a(a_name,is)) { //a_name does not exist as a file or dir.
    if(!create(a_name)) return false;
  } else {
    if(!is) return false; //a_name exists but is not a directory.
  }
  return cd(a_name);
}

inline bool entries(const std::string& a_path,std::vector<std::string>& a_list,bool a_full_path = true){
  a_list.clear();
  struct stat finfo;
  if (::stat(a_path.c_str(),&finfo) < 0)  return false;
#ifdef WIN32
  if (!(finfo.st_mode & S_IFDIR)) return false;
  std::string entry = a_path;
  if (!(entry[entry.size()] == '/' || entry[entry.size()] == '\\' ))
    entry += "\\";
  entry += "*";
  WIN32_FIND_DATA findFileData;
  HANDLE dir = ::FindFirstFile(entry.c_str(),&findFileData);
  if(dir == INVALID_HANDLE_VALUE) return false;
  // Get file names :
  for (;;) {
    if(!::FindNextFile(dir,&findFileData)) break;
    std::string name = (const char*)findFileData.cFileName;
    // Be sure we can work on the file :
    std::string fname = a_path+"\\"+name;
    if (::stat(fname.c_str(),&finfo) < 0)  continue;
    if(a_full_path)
      a_list.push_back(fname);
    else
      a_list.push_back(name);
  }
  ::FindClose(dir);
#else
  if (!S_ISDIR(finfo.st_mode)) return false;
  DIR* dir = ::opendir(a_path.c_str());
  if(!dir) return false;
  // Get file names :
  for (;;) {
    struct dirent* dp = ::readdir(dir);
    //struct direct* dp;
    if (dp==NULL) break;
    if (REAL_DIR_ENTRY(dp)) {
      std::string name = dp->d_name;
      // Be sure we can work on the file :
      std::string fname = a_path+"/"+name;
      if (::stat(fname.c_str(),&finfo) < 0)  continue;
      if(a_full_path)
        a_list.push_back(fname);
      else
        a_list.push_back(name);
    }
  }
  ::closedir(dir);
#endif
  return true;
}

}}

#endif
